#!/usr/bin/env python
#
# cslegion_create_3D_unit_bit_file.py
#
# Converts a 3D unit image (bitmap) to outline format (BIT)
#
# >>> GIMP uses a non-standard BMP file version. Save all resized images to 24-bit with a PSP Batch run. <<<
#
###########################################################
from gimpfu import *
from datetime import datetime
import gimpcolor
import os
import re

###########################################################
#
# Subfunction for creating the outline file
#
def sub_create_bit_outline(orig_img, orig_imgname):
    # Start undo group
    pdb.gimp_image_undo_group_start(orig_img)    
    
    # Set context    
    pdb.gimp_context_set_sample_threshold_int(0) # threshold 0 <-> 255; 0 = must be exactly the same
    pdb.gimp_context_set_opacity(100) # opacity 0 <-> 100; 100 fully opaque
    pdb.gimp_context_set_feather(FALSE) # no feather effect when combining selections
    pdb.gimp_context_set_antialias(FALSE) # no antialias effect for selections either
    pdb.gimp_context_set_sample_merged(FALSE) # seems to be not selected with GIMP by default?
    pdb.gimp_context_set_sample_criterion(SELECT_CRITERION_COMPOSITE) # this is the default anyway
    
    pdb.gimp_message("Step 0 - Housekeeping done") # debug
    
    # 1. Select by Color    
    bckgr_color = pdb.gimp_image_pick_color(orig_img, orig_img.layers[0], 0, 0, FALSE, FALSE, 0) # pick the existing background color from 0,0
    pdb.gimp_image_select_color(orig_img, CHANNEL_OP_REPLACE,  orig_img.layers[0], bckgr_color)
    pdb.gimp_message("Step 1 - Select by Color done") # debug
    
    # 2. Colors-Map-Color Exchange selection to ff00ff
    pdb.gimp_context_set_foreground(gimpcolor.RGB(255, 0, 255)) # set foreground color to magenta (ff00ff)
    # set background color to full white
    pdb.gimp_context_set_background(gimpcolor.RGB(255, 255, 255)) # set background color to white (ffffff)
    pdb.gimp_drawable_edit_fill(orig_img.layers[0], FILL_FOREGROUND) # fill selection with foreground color    
    pdb.gimp_message("Step 2 - Colors Map Exchange done") # debug
    
    # 3. Select-Border, 1px, Border style: hard
    pdb.gimp_selection_border(orig_img, 1) # border selection by 1px
    pdb.gimp_message("Step 3 - Select Border done") # debug

    # 4. Select-Invert
    pdb.gimp_selection_invert(orig_img) # invert the selection
    pdb.gimp_message("Step 4 - Select Invert done") # debug
    # 5. Edit-Clear
    pdb.gimp_edit_clear(orig_img.layers[0]) # delete the selection
    pdb.gimp_selection_none(orig_img) # deselect all
    #pdb.gimp_image_merge_visible_layers(orig_img, CLIP_TO_IMAGE) # merge selection back to image
    pdb.gimp_message("Step 5 - Edit Clear done") # debug
    
    # 6. Filltool hex'ffffff' at 0,0    
    pdb.gimp_context_set_foreground(gimpcolor.RGB(255, 255, 255)) # Set foreground color to white (ffffff)        
    pdb.gimp_image_select_color(orig_img, CHANNEL_OP_REPLACE, orig_img.layers[0], pdb.gimp_image_pick_color(orig_img, orig_img.layers[0], 0, 0, FALSE, FALSE, 0))
    pdb.gimp_drawable_edit_fill(orig_img.layers[0], FILL_FOREGROUND) # fill selection with foreground color    
    pdb.gimp_message("Step 6 - Fill image edges with White done") # debug

    # 7. Exchange color	ff00ff to 000000
    pdb.gimp_context_set_foreground(gimpcolor.RGB(0, 0, 0)) # Set foreground color to black (000000)
    pdb.gimp_image_select_color(orig_img, CHANNEL_OP_REPLACE,  orig_img.layers[0], gimpcolor.RGB(255, 0, 255)) # Select all magenta pixels
    pdb.gimp_drawable_edit_fill(orig_img.layers[0], FILL_FOREGROUND) # fill selection with foreground color
    #pdb.gimp_image_merge_visible_layers(orig_img, CLIP_TO_IMAGE) # merge selection back to image
    pdb.gimp_message("Step 7 - Fill Outline to Black done") # debug

    # 8. Image-Mode-Indexed... Use black and white (1-bit) palette
    pdb.gimp_image_convert_indexed(orig_img, NO_DITHER, MAKE_PALETTE, 2, FALSE, FALSE, "") # convert to 1-bit black and white palette
    pdb.gimp_selection_none(orig_img) # deselect all
    pdb.gimp_message("Step 8 - Image Mode Indexed Black and White (1-bit) Palette done") # debug
    
    # 8. Export as *.bmp    
    # Save outline image under the new name    
    filename3Dbit = orig_imgname[:(len(orig_imgname)-4)] + "_BIT.bmp"
    pdb.gimp_file_save(orig_img, orig_img.layers[0], filename3Dbit, filename3Dbit)
    pdb.gimp_message("Step 9 - Export as *_BIT.bmp done") # debug

    # Set foreground color back to black and background color back to white
    pdb.gimp_context_set_foreground(gimpcolor.RGB(0, 0, 0)) # Set foreground color to black (000000)
    pdb.gimp_context_set_background(gimpcolor.RGB(255, 255, 255)) # Set background color to white (ffffff)    
    
    # End undo group
    pdb.gimp_image_undo_group_end(orig_img)    
    pdb.gimp_message("Step 10 - Foreground and Background default restored") # debug

###########################################################
#
# Plug-in function
#
def cslegion_create_3D_unit_bit_file():    
    pdb.gimp_message("3D Unit Image Outline - Start") # debug
    
    img = gimp.image_list()[0]  # Get the active image
    orig_imgname = img.filename # Get the filename of the active image
    
    sub_create_bit_outline(img, orig_imgname)
    
    pdb.gimp_message("3D Unit Image Outline - Finished")
    pdb.gimp_message("To restore the original image, press <Ctrl+Z> or select Edit -> Undo 5a. Create a 3D Unit Image Outline BIT file")

###########################################################
#
# Register function
#
register(
	"cslegion_create_3D_unit_bit_file",
	"Converts a 3D unit image (bitmap) to outline format (BIT).",
	"Converts a 3D unit image (bitmap) to outline format (BIT).",
	"P. Nieminen",	# author
	"P. Nieminen",	# copyright holder
	"2025",			# date
	"5a. Create a 3D Unit Image Outline BIT file",
	"", 
	[],
	[],
	cslegion_create_3D_unit_bit_file, 
	menu="<Image>/Tools/CS Legion Tools"
) # end register

main()
